@@ -1,12 +1,24 @@
 package Text::Tradition::Collation::Reading;
 
 use Moose;
+use Moose::Util qw/ does_role apply_all_roles /;
+use Moose::Util::TypeConstraints;
+use Text::Tradition::Error;
+use XML::Easy::Syntax qw( $xml10_name_rx $xml10_namestartchar_rx );
 use overload '""' => \&_stringify, 'fallback' => 1;
 
+subtype 'ReadingID',
+	as 'Str',
+	where { $_ =~ /\A$xml10_name_rx\z/ },
+	message { 'Reading ID must be a valid XML attribute string' };
+	
+no Moose::Util::TypeConstraints;
+
 =head1 NAME
 
-Text::Tradition::Collation::Reading - represents a reading (usually a word) in a collation.
-    
+Text::Tradition::Collation::Reading - represents a reading (usually a word)
+in a collation.
+
 =head1 DESCRIPTION
 
 Text::Tradition is a library for representation and analysis of collated
@@ -19,12 +31,13 @@ a particular sequence
 
 =head2 new
 
-Creates a new reading in the given collation with the given attributes. 
+Creates a new reading in the given collation with the given attributes.
 Options include:
 
 =over 4
 
-=item collation - The Text::Tradition::Collation object to which this reading belongs.  Required.
+=item collation - The Text::Tradition::Collation object to which this
+reading belongs.  Required.
 
 =item id - A unique identifier for this reading. Required.
 
@@ -36,9 +49,11 @@ Options include:
 
 =item is_lacuna - The 'reading' represents a known gap in the text.
 
-=item is_ph - A temporary placeholder for apparatus parsing purposes.  Do not use unless you know what you are doing.
+=item is_ph - A temporary placeholder for apparatus parsing purposes.  Do
+not use unless you know what you are doing.
 
-=item rank - The sequence number of the reading. This should probably not be set manually.
+=item rank - The sequence number of the reading. This should probably not
+be set manually.
 
 =back
 
@@ -71,7 +86,7 @@ has 'collation' => (
 
 has 'id' => (
 	is => 'ro',
-	isa => 'Str',
+	isa => 'ReadingID',
 	required => 1,
 	);
 
@@ -82,6 +97,12 @@ has 'text' => (
 	writer => 'alter_text',
 	);
 	
+has 'language' => (
+	is => 'ro',
+	isa => 'Str',
+	predicate => 'has_language',
+	);
+	
 has 'is_start' => (
 	is => 'ro',
 	isa => 'Bool',
@@ -118,19 +139,21 @@ has 'rank' => (
     predicate => 'has_rank',
     clearer => 'clear_rank',
     );
-
+    
 ## For prefix/suffix readings
 
 has 'join_prior' => (
 	is => 'ro',
 	isa => 'Bool',
 	default => undef,
+	writer => '_set_join_prior',
 	);
 	
 has 'join_next' => (
 	is => 'ro',
 	isa => 'Bool',
 	default => undef,
+	writer => '_set_join_next',
 	);
 
 
@@ -146,22 +169,47 @@ around BUILDARGS => sub {
 			
 	# If one of our special booleans is set, we change the text and the
 	# ID to match.
-	if( exists $args->{'is_lacuna'} && !exists $args->{'text'} ) {
+	if( exists $args->{'is_lacuna'} && $args->{'is_lacuna'} && !exists $args->{'text'} ) {
 		$args->{'text'} = '#LACUNA#';
-	} elsif( exists $args->{'is_start'} ) {
-		$args->{'id'} = '#START#';  # Change the ID to ensure we have only one
+	} elsif( exists $args->{'is_start'} && $args->{'is_start'} ) {
+		$args->{'id'} = '__START__';  # Change the ID to ensure we have only one
 		$args->{'text'} = '#START#';
 		$args->{'rank'} = 0;
-	} elsif( exists $args->{'is_end'} ) {
-		$args->{'id'} = '#END#';	# Change the ID to ensure we have only one
+	} elsif( exists $args->{'is_end'} && $args->{'is_end'} ) {
+		$args->{'id'} = '__END__';	# Change the ID to ensure we have only one
 		$args->{'text'} = '#END#';
-	} elsif( exists $args->{'is_ph'} ) {
+	} elsif( exists $args->{'is_ph'} && $args->{'is_ph'} ) {
 		$args->{'text'} = $args->{'id'};
 	}
 	
+	# Backwards compatibility for non-XMLname IDs
+	my $rid = $args->{'id'};
+	$rid =~ s/\#/__/g;
+	$rid =~ s/[\/,]/./g;
+    if( $rid !~ /^$xml10_namestartchar_rx/ ) {
+    	$rid = 'r'.$rid;
+    }
+	$args->{'id'} = $rid;
+	
 	$class->$orig( $args );
 };
 
+# Look for a lexeme-string argument in the build args; if there, pull in the
+# morphology role if possible.
+sub BUILD {
+	my( $self, $args ) = @_;
+	if( exists $args->{'lexemes'} ) {
+		unless( does_role( $self, 'Text::Tradition::Morphology' ) ) {
+			eval { apply_all_roles( $self, 'Text::Tradition::Morphology' ) };
+			if( $@ ) {
+				warn "No morphology package installed; DROPPING lexemes";
+				return;
+			}
+		}
+		$self->_deserialize_lexemes( $args->{'lexemes'} );
+	}
+}
+
 =head2 is_meta
 
 A meta attribute (ha ha), which should be true if any of our 'special'
@@ -175,6 +223,43 @@ sub is_meta {
 	return $self->is_start || $self->is_end || $self->is_lacuna || $self->is_ph;	
 }
 
+=head2 is_identical( $other_reading )
+
+Returns true if the reading is identical to the other reading. The basic test
+is equality of ->text attributes, but this may be wrapped or overridden by 
+extensions.
+
+=cut
+
+sub is_identical {
+	my( $self, $other ) = @_;
+	return $self->text eq $other->text;
+}
+
+=head2 is_combinable
+
+Returns true if the reading may in theory be combined into a multi-reading
+segment within the collation graph. The reading must not be a meta reading,
+and it must not have any relationships in its own right with any others.
+This test may be wrapped or overridden by extensions.
+
+=cut
+
+sub is_combinable {
+	my $self = shift;
+	return undef if $self->is_meta;
+	return !$self->related_readings();
+}
+
+# Not really meant for public consumption. Adopt the text of the other reading
+# into this reading.
+sub _combine {
+	my( $self, $other, $joinstr ) = @_;
+	$self->alter_text( join( $joinstr, $self->text, $other->text ) );
+	# Change this reading to a joining one if necessary
+	$self->_set_join_next( $other->join_next );
+}
+
 =head1 Convenience methods
 
 =head2 related_readings
@@ -223,18 +308,7 @@ sub successors {
 	return map { $self->collation->reading( $_ ) } @succ;
 }
 
-=head2 set_identical( $other_reading)
-
-Backwards compatibility method, to add a transposition relationship
-between $self and $other_reading.  Don't use this.
-
-=cut
-
-sub set_identical {
-	my( $self, $other ) = @_;
-	return $self->collation->add_relationship( $self, $other, 
-		{ 'type' => 'transposition' } );
-}
+## Utility methods
 
 sub _stringify {
 	my $self = shift;
@@ -246,8 +320,14 @@ sub TO_JSON {
 	return $self->text;
 }
 
+sub throw {
+	Text::Tradition::Error->throw( 
+		'ident' => 'Reading error',
+		'message' => $_[0],
+		);
+}
+
 no Moose;
 __PACKAGE__->meta->make_immutable;
 
 1;
-
