@@ -2,7 +2,10 @@ package Text::Tradition::Parser::Tabular;
 
 use strict;
 use warnings;
-use Text::CSV_XS;
+use Encode qw/ decode_utf8 /;
+use Text::CSV;
+use Text::Tradition::Error;
+use TryCatch;
 
 =head1 NAME
 
@@ -28,7 +31,8 @@ Text::Tradition::Parser::Tabular
 
 =head1 DESCRIPTION
 
-Parser module for Text::Tradition to read an alignment table format, such as CSV.
+Parser module for Text::Tradition to read an alignment table format, such as 
+CSV or Excel.
 
 =head1 METHODS
 
@@ -36,15 +40,28 @@ Parser module for Text::Tradition to read an alignment table format, such as CSV
 
 Takes an initialized tradition and a set of options; creates the
 appropriate nodes and edges on the graph, as well as the appropriate
-witness objects.  The $option_hash must contain either a 'file' or a
-'string' argument with the table to be parsed; it may also contain a 
-'sep_char' argument to specify how the fields are separated.
+witness objects.  The $option_hash can contain the following:
 
-The table should have witnesses arranged in columns, with the witness sigla
-in the first row.  Empty cells are interpreted as omissions (and thus
-stemmatologically relevant.) Longer lacunae in the text, to be disregarded
-in cladistic analysis, may be represented by filling the appropriate cells
-with the tag '#LACUNA#'.
+=over
+
+=item * file - Name of file which contains the data
+
+=item * string - A string that itself contains the data. One of 'file' or 
+'string' is required.
+
+=item * sep_char - For plaintext formats, the field separation character.
+Defaults to "\t" (tab-separated); should be "," for comma-separated format.
+
+=item * excel - If the data is in an Excel file, this option should be set
+to 'xls' (for pre-2007 Excel format) or 'xlsx' (for Excel XML format.)
+
+=back
+
+The data should comprise a table with witnesses arranged in columns, with
+the witness sigla in the first row.  Empty cells are interpreted as
+omissions (and thus stemmatologically relevant.) Longer lacunae in the
+text, to be disregarded in cladistic analysis, may be represented by
+filling the appropriate cells with the tag '#LACUNA#'.
 
 If a witness name ends in the collation's ac_label, it will be treated as
 an 'ante-correction' version of the 'main' witness whose sigil it shares.
@@ -108,57 +125,80 @@ foreach my $k ( keys %seen_wits ) {
 	}
 }	
 
+# Check that we only have collation relationships where we need them
+is( scalar $t->collation->relationships, 3, "Redundant collations were removed" );
+
+## Check excel parsing
+
+my $xls = 't/data/armexample.xls';
+my $xt = Text::Tradition->new(
+	'name'  => 'excel test',
+	'input' => 'Tabular',
+	'file'  => $xls,
+	'excel'   => 'xls'
+	);
+
+is( ref( $xt ), 'Text::Tradition', "Parsed test Excel 97-2004 file" );
+my %xls_wits;
+map { $xls_wits{$_} = 0 } qw/ Wit1 Wit2 Wit3 /;
+foreach my $wit ( $xt->witnesses ) {
+	$xls_wits{$wit->sigil} = 1;
+}
+is( scalar keys %xls_wits, 3, "No extra witnesses were made" );
+foreach my $k ( keys %xls_wits ) {
+	ok( $xls_wits{$k}, "Witness $k still exists" );
+}
+is( scalar $xt->collation->readings, 11, "Got correct number of test readings" );
+is( scalar $xt->collation->paths, 13, "Got correct number of reading paths" );
+is( $xt->collation->reading('r5.1')->text, "\x{587}", 
+	"Correct decoding of at least one reading" );
+
+my $xlsx = 't/data/armexample.xlsx';
+my $xtx = Text::Tradition->new(
+	'name'  => 'excel test',
+	'input' => 'Tabular',
+	'file'  => $xlsx,
+	'excel'   => 'xlsx'
+	);
+
+is( ref( $xtx ), 'Text::Tradition', "Parsed test Excel 2007+ file" );
+my %xlsx_wits;
+map { $xlsx_wits{$_} = 0 } qw/ Wit1 Wit2 Wit3 /;
+foreach my $wit ( $xtx->witnesses ) {
+	$xlsx_wits{$wit->sigil} = 1;
+}
+is( scalar keys %xlsx_wits, 3, "No extra witnesses were made" );
+foreach my $k ( keys %xlsx_wits ) {
+	ok( $xlsx_wits{$k}, "Witness $k still exists" );
+}
+is( scalar $xtx->collation->readings, 12, "Got correct number of test readings" );
+is( scalar $xtx->collation->paths, 14, "Got correct number of reading paths" );
+is( $xtx->collation->reading('r5.1')->text, "\x{587}", 
+	"Correct decoding of at least one reading" );
+
 =end testing
 
 =cut
 
 sub parse {
     my( $tradition, $opts ) = @_;
-    my $c = $tradition->collation; # shorthand
-    my $csv_options = { 'binary' => 1 };
-    $csv_options->{'sep_char'} = $opts->{'sep_char'} || "\t";
-    if( $csv_options->{'sep_char'} eq "\t" ) {
-    	# If it is really tab separated, nothing is an escape char.
-    	$csv_options->{'quote_char'} = undef;
-    	$csv_options->{'escape_char'} = undef;
-    }
-    my $csv = Text::CSV_XS->new( $csv_options );
-    
-    my $alignment_table;
-    if( exists $opts->{'string' } ) {
-        my @lines = split( "\n", $opts->{'string'} );
-        foreach my $l ( @lines ) {
-            my $status = $csv->parse( $l );
-            if( $status ) {
-                push( @$alignment_table, [ $csv->fields ] );
-            } else {
-                warn "Could not parse line $l: " . $csv->error_input;
-            }
-        }
-    } elsif( exists $opts->{'file'} ) {
-        open( my $fh, $opts->{'file'} ) 
-            or warn "Could not open input file " . $opts->{'file'};
-        binmode( $fh, ':utf8' );
-        while( my $row = $csv->getline( $fh ) ) {
-            push( @$alignment_table, $row );
-        }
-        close $fh;
-    } else {
-        warn "Could not find string or file option to parse";
-        return;
-    }
-
+    my $alignment_table = _table_from_input( $opts );
     # Set up the witnesses we find in the first line
     my @witnesses;
     my %ac_wits;  # Track layered witness -> main witness mapping
+    my $c = $tradition->collation; # shorthand
+    my $aclabel = $c->ac_label;
     foreach my $sigil ( @{$alignment_table->[0]} ) {
-        my $wit = $tradition->add_witness( 'sigil' => $sigil );
+        if( $sigil =~ /^(.*)\Q$aclabel\E$/ ) {
+        	# Sanitize the sigil name to an XML name
+        	$sigil = $1 . '_layered';
+            $ac_wits{$sigil} = $1;
+        }
+        my $wit = $tradition->add_witness( 
+        	'sigil' => $sigil, 'sourcetype' => 'collation' );
         $wit->path( [ $c->start ] );
         push( @witnesses, $wit );
         my $aclabel = $c->ac_label;
-        if( $sigil =~ /^(.*)\Q$aclabel\E$/ ) {
-            $ac_wits{$sigil} = $1;
-        }
     }
     
     # Save the original witness text sequences. Have to loop back through
@@ -173,11 +213,14 @@ sub parse {
     	$is_layer ? $wit->layertext( \@words ) : $wit->text( \@words );
     }    
     
+    my $nocollate = ( scalar( @witnesses ) * scalar @$alignment_table ) > 150000;
+    print STDERR "Tradition too big for row collation\n" if $nocollate;
+    
     # Now for the next rows, make nodes as necessary, assign their ranks, and 
     # add them to the witness paths.
     foreach my $idx ( 1 .. $#{$alignment_table} ) {
         my $row = $alignment_table->[$idx];
-        my $nodes = _make_nodes( $c, $row, $idx );
+        my $nodes = _make_nodes( $c, $row, $idx, $nocollate );
         foreach my $w ( 0 .. $#{$row} ) {
             # push the appropriate node onto the appropriate witness path
             my $word = $row->[$w];
@@ -244,10 +287,125 @@ sub parse {
 	
 	# Note that our ranks and common readings are set.
 	$c->_graphcalc_done(1);
+	# Remove redundant collation relationships.
+	$c->relations->filter_collations() unless $nocollate;
+}
+
+sub _table_from_input {
+	my $opts = shift;
+	my $alignment_table = [];
+    if( $opts->{'excel'} ) {
+    	my $sheet;
+    	my $need_decode;
+		unless( exists $opts->{'file'} ) {
+			throw( "Must pass the filename for Excel parsing" );
+		}
+    	if( $opts->{'excel'} eq 'xls' ) {
+			try {
+				require Spreadsheet::ParseExcel;
+			} catch {
+				throw( "Need module Spreadsheet::ParseExcel to parse .xls files" );
+			}
+			my $parser = Spreadsheet::ParseExcel->new();
+			my $workbook = $parser->parse( $opts->{'file'} );
+			unless( defined $workbook && defined $workbook->worksheet(0) ) {
+				throw( "Failed to parse file " . $opts->{'file'} . ": " . $parser->error() );
+			}
+			$sheet = $workbook->worksheet(0);
+		} elsif( $opts->{'excel'} eq 'xlsx' ) {
+			try {
+				require Spreadsheet::XLSX;
+			} catch {
+				throw( "Need module Spreadsheet::XLSX to parse .xlsx files" );
+			}
+			$need_decode = 1;
+			my $workbook;
+			try {
+				$workbook = Spreadsheet::XLSX->new( $opts->{'file'} );
+			} catch {
+				throw( "Failed to parse file " . $opts->{'file'} );
+			}
+			$sheet = $workbook->worksheet(0);
+		} else {
+			throw( "Unrecognized Excel variant" . $opts->{'excel'} );
+		}
+		$alignment_table = _alignment_from_worksheet( $sheet, $need_decode );
+    } else {
+    	# Assume it is a comma-, tab-, or whatever-separated format.
+		my $csv_options = { 'binary' => 1 };
+		$csv_options->{'sep_char'} = $opts->{'sep_char'} || "\t";
+		if( $csv_options->{'sep_char'} eq "\t" ) {
+			# If it is really tab separated, nothing is an escape char.
+			$csv_options->{'quote_char'} = undef;
+			$csv_options->{'escape_char'} = undef;
+		}
+		my $csv = Text::CSV->new( $csv_options );
+		
+		if( exists $opts->{'string' } ) {
+			my @lines = split( "\n", $opts->{'string'} );
+			foreach my $l ( @lines ) {
+				my $status = $csv->parse( $l );
+				if( $status ) {
+					push( @$alignment_table, [ $csv->fields ] );
+				} else {
+					throw( "Could not parse line $l: " . $csv->error_input );
+				}
+			}
+		} elsif( exists $opts->{'file'} ) {
+			open( my $fh, $opts->{'file'} ) 
+				or warn "Could not open input file " . $opts->{'file'};
+			binmode( $fh, ':utf8' );
+			while( my $row = $csv->getline( $fh ) ) {
+				push( @$alignment_table, $row );
+			}
+			close $fh;
+		} else {
+			throw( "Could not find string or file option to parse" );
+		}
+	}
+	return $alignment_table;
+}
+sub _alignment_from_worksheet {
+	my( $sheet, $decode ) = @_;
+	my $alignment_table = [];
+	
+	my( $rmin, $rmax ) = $sheet->row_range();
+	my( $cmin, $cmax ) = $sheet->col_range();
+	unless( $cmax && $rmax ) {
+		throw( "Found no rows or no columns in first worksheet" );
+	}
+	# Populate the alignment table. We only want columns that have
+	# a sigil in row zero.
+	my %sigcols = ();
+	push( @$alignment_table, [] );
+	foreach my $col ( $cmin .. $cmax ) {
+		my $cell = $sheet->get_cell( $rmin, $col );
+		my $cellval = $cell ? $cell->value() : undef;
+		if( $cellval ) {
+			$sigcols{$col} = 1;
+			push( @{$alignment_table->[0]}, $cellval );
+		}
+	}
+	# Now go through the rest of the rows and pick up the columns
+	# that were headed by a sigil.
+	foreach my $row ( $rmin+1 .. $rmax ) {
+		my @tablerow;
+		foreach my $col ( $cmin .. $cmax ) {
+			next unless $sigcols{$col};
+			my $cell = $sheet->get_cell( $row, $col );
+			my $cellval;
+			if( $cell ) {
+				$cellval = $decode ? decode_utf8( $cell->value ) : $cell->value;
+			}
+			push( @tablerow, $cellval );
+		}
+		push( @$alignment_table, \@tablerow );
+	}
+	return $alignment_table;
 }
 
 sub _make_nodes {
-    my( $collation, $row, $index ) = @_;
+    my( $collation, $row, $index, $nocollate ) = @_;
     my %unique;
     my $commonctr = 0; # Holds the number of unique readings + gaps, ex. lacunae.
     foreach my $w ( @$row ) {
@@ -257,7 +415,7 @@ sub _make_nodes {
     my $ctr = 1;
     foreach my $w ( keys %unique ) {
     	my $rargs = {
-    		'id' => "$index,$ctr",
+    		'id' => "r$index.$ctr",
     		'rank' => $index,
     		'text' => $w,
     		};
@@ -270,9 +428,36 @@ sub _make_nodes {
         $unique{$w} = $r;
         $ctr++;
     }
+    # Collate this sequence of readings via a single 'collation' relationship.
+    unless( $nocollate ) {
+		my @rankrdgs = values %unique;
+		my $collation_rel;
+		while( @rankrdgs ) {
+			my $r = shift @rankrdgs;
+			next if $r->is_meta;
+			foreach my $nr ( @rankrdgs ) {
+				next if $nr->is_meta;
+				if( $collation_rel ) {
+					$collation->add_relationship( $r, $nr, $collation_rel );
+				} else {
+					$collation->add_relationship( $r, $nr, 
+						{ 'type' => 'collated', 
+						  'annotation' => "Parsed together for rank $index" } );
+					$collation_rel = $collation->get_relationship( $r, $nr );
+				}
+			}
+		}
+	}    
     return \%unique;
 }
 
+sub throw {
+	Text::Tradition::Error->throw( 
+		'ident' => 'Parser::Tabular error',
+		'message' => $_[0],
+		);
+}
+
 1;
 
 =head1 LICENSE
